"""
Copyright (c) 2026 Proton AG

This file is part of Proton VPN.

Proton VPN is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Proton VPN is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ProtonVPN.  If not, see <https://www.gnu.org/licenses/>.
"""
import tempfile

import pytest

from proton.vpn.core.settings.packet_capture import (
    PacketCapture, PacketCaptureMode,
    DEFAULT_DIRECTORY_PATH, DEFAULT_MODE, MAX_PACKET_CAPTURE_FILE_SIZE,
)


def test_default_directory_path():
    assert DEFAULT_DIRECTORY_PATH == tempfile.gettempdir()


def test_default_values():
    pc = PacketCapture.default()
    assert pc.directory_path == DEFAULT_DIRECTORY_PATH
    assert pc.mode == PacketCaptureMode.OVERWRITE
    assert pc.max_bytes == MAX_PACKET_CAPTURE_FILE_SIZE
